# Author: Matthieu Domenech
# Script creating the pomp object encoding the isolation model described in the main text
###############################################################################

# Creates the shared object (.so on Unix systems, .dll on Windows systems)
model <- "isolmodel"
modelfile <- paste(model,".c",sep="")
solib <- paste0(model,.Platform$dynlib.ext)
cmd <- paste(R.home("bin/R"),"CMD SHLIB -o",solib,modelfile)
rv <- system(cmd)
if(rv!=0) stop("cannot compile shared-object library ",solib)

# Creates the pomp object
isolmodel <- pomp(
  data = data.frame(time=seq(from=1, by=7, length=100), PV_OBS=NA, INC_OBS=NA),
  times="time",
  t0=0,
  rprocess=gillespie.sim(rate.fun="rSim",
                         v=cbind(sc1=c(+1,0,0,0), sc2=c(0,+1,0,0),
                                 si=c(0,0,+1,0), c1s=c(-1,0,0,0),
                                 c1i=c(-1,0,+1,0), c2s=c(0,-1,0,0),
                                 c2c1=c(+1,-1,0,0), c2idis=c(0,-1,+1,0), 
                                 c2iacq=c(0,-1,+1,+1), is=c(0,0,-1,0), ic1=c(+1,0,-1,0)),
                         d=cbind(sc1=c(1,1,1,0), acq=c(1,1,1,0),
                                 si=c(1,1,1,0),c1s=c(1,0,0,0),
                                 c1i=c(1,0,1,0), c2s=c(0,1,0,0),
                                 c2c1=c(0,1,0,0),c2idis=c(0,1,1,0),
                                 c2iacq=c(0,1,1,0), is=c(0,0,1,0),ic1=c(0,0,1,0))),
  rmeasure="rObs", 
  dmeasure="dObs",
  PACKAGE="isolmodel",
  obsnames=c("PV_OBS","INC_OBS"),
  zeronames=c("INC"),
  statenames=c("C1","C2","I","INC"),
  paramnames=c("b0","b1","b2","d1","d2","mu_s","mu_c","mu_i","sigma","rho","Ni","N","eps","C1.0","C2.0","I.0"),
  log.trans=c("b0","b1","b2","d1","d2","mu_s","mu_c","mu_i","Ni","N"),
  logit.trans=c("sigma","rho","eps","C1.0","C2.0","I.0"),
  parameter.transform = function(params, log.trans, logit.trans,...){
    params[logit.trans] <- plogis(params[logit.trans])
    params[log.trans] <- exp(params[log.trans])
    params
  },
  parameter.inv.transform = function(params, log.trans, logit.trans,...){
    params[logit.trans] <- qlogis(params[logit.trans])
    params[log.trans] <- log(params[log.trans])
    params
  },
  initializer=function(params, t0, ic.pars, comp.names,...){
    x0 <- c(C1=0, C2=0, I=0, INC=0)
    N <- params["N"]
    fracs <- params[ic.pars]
    x0[comp.names] <- round(N * fracs)
    x0
  },
  ic.pars=c("C1.0","C2.0","I.0"),
  comp.names=c("C1","C2","I")
)

# Loads the shared object
dyn.load(solib)
rm(model, modelfile, rv, cmd, solib)