# Author: Matthieu Domenech
# Main script: sets model parameters, simulates 100 weeks of data
# Performs inference on these simulated data using the iterated filtering algorithm
###############################################################################

rm(list=ls())

#  Load the pomp package, uncomment the following line if not installed
# install.packages("pomp")
library(pomp)

# Sources the script creating the pomp model and the profile loglike function
# The working directory should be changed to the path of the extracted directory (ESM2)
setwd("~/Desktop/ESM2")
source("create_isolmodel.R")
source("profile_loglike.R")

#  Sets model parameters, see Table 1 for description
N <- 16L
parms <- c(b0=1e-2, b1=5e-3, b2=1e-3, d1=1/2, d2=1/4,
           mu_s=1/5, mu_c=1/5, mu_i=1/8, sigma=0.2,
           rho=1, Ni=16L, N=N, eps=0, C1.0=1/N, C2.0=1/N, I.0=2/N)
coef(isolmodel) <- parms

# Simulates model and plots results
# Seed for random number generation is fixed so that results are replicable
isolmodel <- simulate(isolmodel, seed = 2186)
plot(isolmodel)

# # Inference on the simulated data using the profile log likelihood
# # The estimation procedure may take a long time, so the code below is commented by default
# 
# # Choice of estimated parameters and tested ranges of values, see the 'readme.docx' file for details
# estpars <- c("b0", "b1", "b2", "sigma")
# range_estpars <- data.frame(b0=c(0,0.03), b1=c(0,0.03), b2=c(0,0.01), sigma=c(0.1, 0.3))
# 
# # Calls the profile log likelihood function on each parameter with the prespecified range
# # Algorithmic parameters are detailed in the 'readme.docx' file
# # A profiling procedure took ~20 hours per parameter on a desktop computer, so this may take a while
# 
# for(prof_par in estpars){
#   prof <- profile_loglike(prof_par=prof_par, range=range_estpars[, prof_par], 
#                           no_points=50, no_est=2, estpars=estpars, Nmif=60, Np=2e3, var.factor=5)
#   saveRDS(prof, file = paste0(prof_par, ".rds"))
# }